function [BTNSPLHparam time] = trainBTNSPLH(Z_train, Z_vad, meanZ_train, S, BTNSPLHparam)
%BTNSPLH Bootstrap-NSPLH
%
%	Usage:
%       [BTNSPLHparam time] = trainBTNSPLH(Z_train, Z_vad, meanZ_train, S, BTNSPLHparam)
% 
%             Input:
%               Z_train  - Nonlinear transformed data matrix. Each row vector is a data point.
%               Z_vad    - Nonlinear transformed labeled data matrix. Each row vector is a data point.
%               meanZ_train  - mean of each dimension.
%               S          - Pairwise label matrix corresponding to the order of Z_vad. 
%               BTNSPLHparam.nbits  -  Bit length
%               BTNSPLHparam.lambda -  weight for regularizer
%               BTNSPLHparam.eta    -  update weight for \deltaS
%               BTNSPLHparam.alpha  -  update threshold for positive pair
%               BTNSPLHparam.beta   -  update threshold for negative pair
%               BTNSPLHparam.r      -  number of anchors
%               *Z_train and Z_vad are with zeromean
%
%             Output:
%               BTNSPLHparam.W      -  Projection matrix
%               BTNSPLHparam.t      -  Threshold vector
%               time                -  Time cost
%
%   Written by Chenxia Wu (chenxiawu AT hotmail.com)

start_t=clock;
Nvad=size(Z_vad,1);
nbits=BTNSPLHparam.nbits;
r=BTNSPLHparam.r;

M1=Z_vad'*S*Z_vad;
M2=Z_train'*Z_train*BTNSPLHparam.lambda;
clear Z_train
M=M1+M2;
M=(M+M')/2;

W=zeros(r,nbits);
H=zeros(Nvad,Nvad);
pS=S;
pS(pS<0)=0;
nS=S;
nS(nS>0)=0;
S1=S;
for k=1:nbits
    opts.disp=0;
    [eigvector eigvalue]=eigs(M,1,'la',opts);
    W(:,k)=eigvector;
    
    proj = Z_vad*eigvector;
    H=H+sign(proj)*sign(proj)';
    
    pH=BTNSPLHparam.alpha*k-H;
    nH=BTNSPLHparam.beta*k-H;
    pH(pH<0)=0;
    nH(nH>0)=0;
    deltaS=pH.*pS-nH.*nS;
    
    S=S1+BTNSPLHparam.eta*deltaS/(2*k);
    M1=Z_vad'*S*Z_vad;
    du=eye(r,r)-eigvector*eigvector';
    M2=du*M2*du';
    M=M1+M2;
    M=(M+M')/2;
end
BTNSPLHparam.W=W;
BTNSPLHparam.t=meanZ_train*W;
end_t=clock;
time=etime(end_t,start_t);