function [BTSPLHparam time] = trainBTSPLH(fea_train, fea_vad, S, BTSPLHparam)
%BTSPLH Bootstrap-SPLH
%
%	Usage:
%       [BTSPLHparam time] = trainBTSPLH(fea_train, fea_vad, S, BTSPLHparam)
% 
%             Input:
%               fea_train  - Data matrix. Each row vector is a data point.
%               fea_vad    - Labeled data matrix. Each row vector is a data point.
%               S          - Pairwise label matrix corresponding to the order of fea_vad. 
%               BTSPLHparam.nbits  -  Bit length
%               BTSPLHparam.lambda -  weight for regularizer
%               BTSPLHparam.eta    -  update weight for \deltaS
%               BTSPLHparam.alpha  -  update threshold for positive pair
%               BTSPLHparam.beta   -  update threshold for negative pair
%
%             Output:
%               BTSPLHparam.W      -  Projection matrix
%               BTSPLHparam.t      -  Threshold vector
%               time               -  Time cost
%
%   Written by Chenxia Wu (chenxiawu AT hotmail.com)

start_t=clock;
nbits=BTSPLHparam.nbits;
[Nvad dim]=size(fea_vad);

% zero mean
meanfea=mean(fea_train);
% fea_train=fea_train-repmat(meanfea,Ntrain,1);
fea_train=zeromean(fea_train);
fea_vad=fea_vad-repmat(meanfea,Nvad,1);

M1=fea_vad'*S*fea_vad;
M2=fea_train'*fea_train*BTSPLHparam.lambda;
clear fea_train
M=M1+M2;
M=(M+M')/2;

W=zeros(dim,nbits);
H=zeros(Nvad,Nvad);
pS=S;
pS(pS<0)=0;
nS=S;
nS(nS>0)=0;
S1=S;
for k=1:nbits
    opts.disp=0;
    [eigvector eigvalue]=eigs(M,1,'la',opts);
    W(:,k)=eigvector;
    proj = fea_vad*eigvector;
    H=H+sign(proj)*sign(proj)';
    
    pH=BTSPLHparam.alpha*k-H;
    nH=BTSPLHparam.beta*k-H;
    pH(pH<0)=0;
    nH(nH>0)=0;
    deltaS=pH.*pS-nH.*nS;
    
    S=S1+BTSPLHparam.eta*deltaS/(2*k);
    M1=fea_vad'*S*fea_vad;
    du=eye(dim,dim)-eigvector*eigvector';
    M2=du*M2*du';
    M=M1+M2;
    M=(M+M')/2;
end
BTSPLHparam.W=W;
BTSPLHparam.t=meanfea*W;
end_t=clock;
time=etime(end_t,start_t);